﻿using System;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using OpcLabs.BaseLib;
using OpcLabs.UnitTestLib;

namespace OpcLabs.BaseLibTest
{
    //[TestClass]
    public partial class ObjectDescriptorTest
    {
        #region Private Constants

        private const string TestClsidString = "0530098F-9B36-4049-BC96-AA8DB5F9399F";

        private const string TestClsidServerClass = "{" + TestClsidString + "}";

        #endregion



        #region Static Private Properties

        static private Guid TestClsid => Guid.Parse(TestClsidString);

        #endregion



        #region Test Methods

        [TestMethod]
        public void CtorTest_String_Empty()
        {
            var target = new ObjectDescriptor("");
            Assert.AreEqual("", target.ToString());
            Assert.AreEqual("dcom", target.Scheme);
            Assert.AreEqual("", target.MachineName);
            Assert.AreEqual("", target.ProgId);
            Assert.AreEqual("", target.ServerClass);
            Assert.AreEqual("", target.ObjectId);
            Assert.AreEqual("", target.ClsidString);
            Assert.AreEqual(Guid.Empty, target.Clsid);
        }

        [TestMethod]
        public void CtorTest_String_NoScheme_NoAuthority_Path1()
        {
            var target = new ObjectDescriptor("testSegment");
            Assert.AreEqual("dcom:testSegment", target.ToString());
            Assert.AreEqual("dcom", target.Scheme);
            Assert.AreEqual("", target.MachineName);
            Assert.AreEqual("testSegment", target.ProgId);
            Assert.AreEqual("testSegment", target.ServerClass);
            Assert.AreEqual("testSegment", target.ObjectId);
            Assert.AreEqual("", target.ClsidString);
            Assert.AreEqual(Guid.Empty, target.Clsid);
        }

        [TestMethod]
        public void CtorTest_String_NoScheme_NoAuthority_Path1_Clsid()
        {
            var target = new ObjectDescriptor(TestClsidServerClass);
            Assert.AreEqual("dcom:" + TestClsidServerClass, target.ToString());
            Assert.AreEqual("dcom", target.Scheme);
            Assert.AreEqual("", target.MachineName);
            Assert.AreEqual("", target.ProgId);
            Assert.AreEqual(TestClsidServerClass.ToLower(), target.ServerClass);
            Assert.AreEqual(TestClsidServerClass.ToLower(), target.ObjectId);
            Assert.AreEqual(TestClsidString.ToLower(), target.ClsidString);
            Assert.AreEqual(TestClsid, target.Clsid);
        }

        [TestMethod]
        public void CtorTest_String_NoScheme_NoAuthority_Path2()
        {
            var target = new ObjectDescriptor("/testSegment");
            Assert.AreEqual("dcom:/testSegment", target.ToString());
            Assert.AreEqual("dcom", target.Scheme);
            Assert.AreEqual("", target.MachineName);
            Assert.AreEqual("testSegment", target.ProgId);
            Assert.AreEqual("testSegment", target.ServerClass);
            Assert.AreEqual("testSegment", target.ObjectId);
            Assert.AreEqual("", target.ClsidString);
            Assert.AreEqual(Guid.Empty, target.Clsid);
        }

        [TestMethod]
        public void CtorTest_String_NoScheme_NoAuthority_Path2_Clsid()
        {
            var target = new ObjectDescriptor("/" + TestClsidServerClass);
            Assert.AreEqual("dcom:/" + TestClsidServerClass, target.ToString());
            Assert.AreEqual("dcom", target.Scheme);
            Assert.AreEqual("", target.MachineName);
            Assert.AreEqual("", target.ProgId);
            Assert.AreEqual(TestClsidServerClass.ToLower(), target.ServerClass);
            Assert.AreEqual(TestClsidServerClass.ToLower(), target.ObjectId);
            Assert.AreEqual(TestClsidString.ToLower(), target.ClsidString);
            Assert.AreEqual(TestClsid, target.Clsid);
        }

        [TestMethod]
        public void CtorTest_String_NoScheme_NoAuthority_Path3()
        {
            var target = new ObjectDescriptor("testSegment1/testSegment2");
            Assert.AreEqual("dcom:testSegment1/testSegment2", target.ToString());
            Assert.AreEqual("dcom", target.Scheme);
            Assert.AreEqual("", target.MachineName);
            Assert.AreEqual("testSegment1", target.ProgId);
            Assert.AreEqual("testSegment1", target.ServerClass);
            Assert.AreEqual("testSegment1", target.ObjectId);
            Assert.AreEqual("", target.ClsidString);
            Assert.AreEqual(Guid.Empty, target.Clsid);
        }

        [TestMethod]
        public void CtorTest_String_NoScheme_NoAuthority_Path3_Clsid()
        {
            var target = new ObjectDescriptor("testSegment1/" + TestClsidServerClass);
            Assert.AreEqual("dcom:testSegment1/" + TestClsidServerClass, target.ToString());
            Assert.AreEqual("dcom", target.Scheme);
            Assert.AreEqual("", target.MachineName);
            Assert.AreEqual("testSegment1", target.ProgId);
            Assert.AreEqual("testSegment1", target.ServerClass);
            Assert.AreEqual("testSegment1/" + TestClsidServerClass.ToLower(), target.ObjectId);
            Assert.AreEqual(TestClsidString.ToLower(), target.ClsidString);
            Assert.AreEqual(TestClsid, target.Clsid);
        }

        [TestMethod]
        public void CtorTest_String_NoScheme_NoAuthority_Path4()
        {
            var target = new ObjectDescriptor("/testSegment1/testSegment2");
            Assert.AreEqual("dcom:/testSegment1/testSegment2", target.ToString());
            Assert.AreEqual("dcom", target.Scheme);
            Assert.AreEqual("", target.MachineName);
            Assert.AreEqual("testSegment1", target.ProgId);
            Assert.AreEqual("testSegment1", target.ServerClass);
            Assert.AreEqual("testSegment1", target.ObjectId);
            Assert.AreEqual("", target.ClsidString);
            Assert.AreEqual(Guid.Empty, target.Clsid);
        }

        [TestMethod]
        public void CtorTest_String_NoScheme_NoAuthority_Path4_Clsid()
        {
            var target = new ObjectDescriptor("/testSegment1/" + TestClsidServerClass);
            Assert.AreEqual("dcom:/testSegment1/" + TestClsidServerClass, target.ToString());
            Assert.AreEqual("dcom", target.Scheme);
            Assert.AreEqual("", target.MachineName);
            Assert.AreEqual("testSegment1", target.ProgId);
            Assert.AreEqual("testSegment1", target.ServerClass);
            Assert.AreEqual("testSegment1/" + TestClsidServerClass.ToLower(), target.ObjectId);
            Assert.AreEqual(TestClsidString.ToLower(), target.ClsidString);
            Assert.AreEqual(TestClsid, target.Clsid);
        }

        [TestMethod]
        public void CtorTest_String_NoScheme_NoAuthority_Path5()
        {
            var target = new ObjectDescriptor("/testSegment1/testSegment2/testSegment3");
            Assert.AreEqual("dcom:/testSegment1/testSegment2/testSegment3", target.ToString());
            Assert.AreEqual("dcom", target.Scheme);
            Assert.AreEqual("", target.MachineName);
            Assert.AreEqual("testSegment1", target.ProgId);
            Assert.AreEqual("testSegment1", target.ServerClass);
            Assert.AreEqual("testSegment1", target.ObjectId);
            Assert.AreEqual("", target.ClsidString);
            Assert.AreEqual(Guid.Empty, target.Clsid);
        }

        [TestMethod]
        public void CtorTest_String_NoScheme_NoAuthority_Path5_Clsid()
        {
            var target = new ObjectDescriptor("/testSegment1/" + TestClsidServerClass + "/testSegment3");
            Assert.AreEqual("dcom:/testSegment1/" + TestClsidServerClass + "/testSegment3", target.ToString());
            Assert.AreEqual("dcom", target.Scheme);
            Assert.AreEqual("", target.MachineName);
            Assert.AreEqual("testSegment1", target.ProgId);
            Assert.AreEqual("testSegment1", target.ServerClass);
            Assert.AreEqual("testSegment1/" + TestClsidServerClass.ToLower(), target.ObjectId);
            Assert.AreEqual(TestClsidString.ToLower(), target.ClsidString);
            Assert.AreEqual(TestClsid, target.Clsid);
        }

        [TestMethod]
        public void CtorTest_String_NoScheme_Authority_NoPath()
        {
            var target = new ObjectDescriptor("//testAuthority");
            Assert.AreEqual("dcom://testAuthority", target.ToString());
            Assert.AreEqual("dcom", target.Scheme);
            Assert.AreEqual("testauthority", target.MachineName);
            Assert.AreEqual("", target.ProgId);
            Assert.AreEqual("", target.ServerClass);
            Assert.AreEqual("", target.ObjectId);
            Assert.AreEqual("", target.ClsidString);
            Assert.AreEqual(Guid.Empty, target.Clsid);
        }

        [TestMethod]
        public void CtorTest_String_NoScheme_Authority_Path1()
        {
            var target = new ObjectDescriptor("//testAuthority/testSegment");
            Assert.AreEqual("dcom://testAuthority/testSegment", target.ToString());
            Assert.AreEqual("dcom", target.Scheme);
            Assert.AreEqual("testauthority", target.MachineName);
            Assert.AreEqual("testSegment", target.ProgId);
            Assert.AreEqual("testSegment", target.ServerClass);
            Assert.AreEqual("testSegment", target.ObjectId);
            Assert.AreEqual("", target.ClsidString);
            Assert.AreEqual(Guid.Empty, target.Clsid);
        }

        [TestMethod]
        public void CtorTest_String_NoScheme_Authority_Path1_Clsid()
        {
            var target = new ObjectDescriptor("//testAuthority/" + TestClsidServerClass);
            Assert.AreEqual("dcom://testAuthority/" + TestClsidServerClass, target.ToString());
            Assert.AreEqual("dcom", target.Scheme);
            Assert.AreEqual("testauthority", target.MachineName);
            Assert.AreEqual("", target.ProgId);
            Assert.AreEqual(TestClsidServerClass.ToLower(), target.ServerClass);
            Assert.AreEqual(TestClsidServerClass.ToLower(), target.ObjectId);
            Assert.AreEqual(TestClsidString.ToLower(), target.ClsidString);
            Assert.AreEqual(TestClsid, target.Clsid);
        }

        [TestMethod]
        public void CtorTest_String_Scheme_NoAuthority_Path1()
        {
            var target = new ObjectDescriptor("testScheme:testSegment");
            Assert.AreEqual("testScheme:testSegment", target.ToString());
            Assert.AreEqual("testscheme", target.Scheme);
            Assert.AreEqual("", target.MachineName);
            Assert.AreEqual("testSegment", target.ProgId);
            Assert.AreEqual("testSegment", target.ServerClass);
            Assert.AreEqual("testSegment", target.ObjectId);
            Assert.AreEqual("", target.ClsidString);
            Assert.AreEqual(Guid.Empty, target.Clsid);
        }

        [TestMethod]
        public void CtorTest_String_Scheme_NoAuthority_Path1_Clsid()
        {
            var target = new ObjectDescriptor("testScheme:" + TestClsidServerClass);
            Assert.AreEqual("testScheme:" + TestClsidServerClass, target.ToString());
            Assert.AreEqual("testscheme", target.Scheme);
            Assert.AreEqual("", target.MachineName);
            Assert.AreEqual("", target.ProgId);
            Assert.AreEqual(TestClsidServerClass.ToLower(), target.ServerClass);
            Assert.AreEqual(TestClsidServerClass.ToLower(), target.ObjectId);
            Assert.AreEqual(TestClsidString.ToLower(), target.ClsidString);
            Assert.AreEqual(TestClsid, target.Clsid);
        }

        [TestMethod]
        public void CtorTest_String_Scheme_NoAuthority_Path2()
        {
            var target = new ObjectDescriptor("testScheme:/testSegment");
            Assert.AreEqual("testScheme:/testSegment", target.ToString());
            Assert.AreEqual("testscheme", target.Scheme);
            Assert.AreEqual("", target.MachineName);
            Assert.AreEqual("testSegment", target.ProgId);
            Assert.AreEqual("testSegment", target.ServerClass);
            Assert.AreEqual("testSegment", target.ObjectId);
            Assert.AreEqual("", target.ClsidString);
            Assert.AreEqual(Guid.Empty, target.Clsid);
        }

        [TestMethod]
        public void CtorTest_String_Scheme_NoAuthority_Path2_Clsid()
        {
            var target = new ObjectDescriptor("testScheme:/" + TestClsidServerClass);
            Assert.AreEqual("testScheme:/" + TestClsidServerClass, target.ToString());
            Assert.AreEqual("testscheme", target.Scheme);
            Assert.AreEqual("", target.MachineName);
            Assert.AreEqual("", target.ProgId);
            Assert.AreEqual(TestClsidServerClass.ToLower(), target.ServerClass);
            Assert.AreEqual(TestClsidServerClass.ToLower(), target.ObjectId);
            Assert.AreEqual(TestClsidString.ToLower(), target.ClsidString);
            Assert.AreEqual(TestClsid, target.Clsid);
        }

        [TestMethod]
        public void CtorTest_String_Scheme_NoAuthority_Path3()
        {
            var target = new ObjectDescriptor("testScheme:testSegment1/testSegment2");
            Assert.AreEqual("testScheme:testSegment1/testSegment2", target.ToString());
            Assert.AreEqual("testscheme", target.Scheme);
            Assert.AreEqual("", target.MachineName);
            Assert.AreEqual("testSegment1", target.ProgId);
            Assert.AreEqual("testSegment1", target.ServerClass);
            Assert.AreEqual("testSegment1", target.ObjectId);
            Assert.AreEqual("", target.ClsidString);
            Assert.AreEqual(Guid.Empty, target.Clsid);
        }

        [TestMethod]
        public void CtorTest_String_Scheme_NoAuthority_Path3_Clsid()
        {
            var target = new ObjectDescriptor("testScheme:testSegment1/" + TestClsidServerClass);
            Assert.AreEqual("testScheme:testSegment1/" + TestClsidServerClass, target.ToString());
            Assert.AreEqual("testscheme", target.Scheme);
            Assert.AreEqual("", target.MachineName);
            Assert.AreEqual("testSegment1", target.ProgId);
            Assert.AreEqual("testSegment1", target.ServerClass);
            Assert.AreEqual("testSegment1/" + TestClsidServerClass.ToLower(), target.ObjectId);
            Assert.AreEqual(TestClsidString.ToLower(), target.ClsidString);
            Assert.AreEqual(TestClsid, target.Clsid);
        }

        [TestMethod]
        public void CtorTest_String_Scheme_NoAuthority_Path4()
        {
            var target = new ObjectDescriptor("testScheme:/testSegment1/testSegment2");
            Assert.AreEqual("testScheme:/testSegment1/testSegment2", target.ToString());
            Assert.AreEqual("testscheme", target.Scheme);
            Assert.AreEqual("", target.MachineName);
            Assert.AreEqual("testSegment1", target.ProgId);
            Assert.AreEqual("testSegment1", target.ServerClass);
            Assert.AreEqual("testSegment1", target.ObjectId);
            Assert.AreEqual("", target.ClsidString);
            Assert.AreEqual(Guid.Empty, target.Clsid);
        }

        [TestMethod]
        public void CtorTest_String_Scheme_NoAuthority_Path4_Clsid()
        {
            var target = new ObjectDescriptor("testScheme:/testSegment1/" + TestClsidServerClass);
            Assert.AreEqual("testScheme:/testSegment1/" + TestClsidServerClass, target.ToString());
            Assert.AreEqual("testscheme", target.Scheme);
            Assert.AreEqual("", target.MachineName);
            Assert.AreEqual("testSegment1", target.ProgId);
            Assert.AreEqual("testSegment1", target.ServerClass);
            Assert.AreEqual("testSegment1/" + TestClsidServerClass.ToLower(), target.ObjectId);
            Assert.AreEqual(TestClsidString.ToLower(), target.ClsidString);
            Assert.AreEqual(TestClsid, target.Clsid);
        }

        [TestMethod]
        public void CtorTest_String_Scheme_NoAuthority_Path5()
        {
            var target = new ObjectDescriptor("testScheme:testSegment1/testSegment2/testSegment3");
            Assert.AreEqual("testScheme:testSegment1/testSegment2/testSegment3", target.ToString());
            Assert.AreEqual("testscheme", target.Scheme);
            Assert.AreEqual("", target.MachineName);
            Assert.AreEqual("testSegment1", target.ProgId);
            Assert.AreEqual("testSegment1", target.ServerClass);
            Assert.AreEqual("testSegment1", target.ObjectId);
            Assert.AreEqual("", target.ClsidString);
            Assert.AreEqual(Guid.Empty, target.Clsid);
        }

        [TestMethod]
        public void CtorTest_String_Scheme_NoAuthority_Path5_Clsid()
        {
            var target = new ObjectDescriptor("testScheme:testSegment1/" + TestClsidServerClass + "/testSegment3");
            Assert.AreEqual("testScheme:testSegment1/" + TestClsidServerClass + "/testSegment3", target.ToString());
            Assert.AreEqual("testscheme", target.Scheme);
            Assert.AreEqual("", target.MachineName);
            Assert.AreEqual("testSegment1", target.ProgId);
            Assert.AreEqual("testSegment1", target.ServerClass);
            Assert.AreEqual("testSegment1/" + TestClsidServerClass.ToLower(), target.ObjectId);
            Assert.AreEqual(TestClsidString.ToLower(), target.ClsidString);
            Assert.AreEqual(TestClsid, target.Clsid);
        }

        [TestMethod]
        public void CtorTest_String_Scheme_NoAuthority_Path6()
        {
            var target = new ObjectDescriptor("testScheme:/testSegment1/testSegment2/testSegment3");
            Assert.AreEqual("testScheme:/testSegment1/testSegment2/testSegment3", target.ToString());
            Assert.AreEqual("testscheme", target.Scheme);
            Assert.AreEqual("", target.MachineName);
            Assert.AreEqual("testSegment1", target.ProgId);
            Assert.AreEqual("testSegment1", target.ServerClass);
            Assert.AreEqual("testSegment1", target.ObjectId);
            Assert.AreEqual("", target.ClsidString);
            Assert.AreEqual(Guid.Empty, target.Clsid);
        }

        [TestMethod]
        public void CtorTest_String_Scheme_NoAuthority_Path6_Clsid()
        {
            var target = new ObjectDescriptor("testScheme:/testSegment1/" + TestClsidServerClass + "/testSegment3");
            Assert.AreEqual("testScheme:/testSegment1/" + TestClsidServerClass + "/testSegment3", target.ToString());
            Assert.AreEqual("testscheme", target.Scheme);
            Assert.AreEqual("", target.MachineName);
            Assert.AreEqual("testSegment1", target.ProgId);
            Assert.AreEqual("testSegment1", target.ServerClass);
            Assert.AreEqual("testSegment1/" + TestClsidServerClass.ToLower(), target.ObjectId);
            Assert.AreEqual(TestClsidString.ToLower(), target.ClsidString);
            Assert.AreEqual(TestClsid, target.Clsid);
        }

        [TestMethod]
        public void CtorTest_String_Scheme_Authority_Path1()
        {
            var target = new ObjectDescriptor("testScheme://testAuthority/testSegment");
            Assert.AreEqual("testScheme://testAuthority/testSegment", target.ToString());
            Assert.AreEqual("testscheme", target.Scheme);
            Assert.AreEqual("testauthority", target.MachineName);
            Assert.AreEqual("testSegment", target.ProgId);
            Assert.AreEqual("testSegment", target.ServerClass);
            Assert.AreEqual("testSegment", target.ObjectId);
            Assert.AreEqual("", target.ClsidString);
            Assert.AreEqual(Guid.Empty, target.Clsid);
        }

        [TestMethod]
        public void CtorTest_String_Scheme_Authority_Path1_Clsid()
        {
            var target = new ObjectDescriptor("testScheme://testAuthority/" + TestClsidServerClass);
            Assert.AreEqual("testScheme://testAuthority/" + TestClsidServerClass, target.ToString());
            Assert.AreEqual("testscheme", target.Scheme);
            Assert.AreEqual("testauthority", target.MachineName);
            Assert.AreEqual("", target.ProgId);
            Assert.AreEqual(TestClsidServerClass.ToLower(), target.ServerClass);
            Assert.AreEqual(TestClsidServerClass.ToLower(), target.ObjectId);
            Assert.AreEqual(TestClsidString.ToLower(), target.ClsidString);
            Assert.AreEqual(TestClsid, target.Clsid);
        }

        [TestMethod]
        public void CtorTest_String_Scheme_Authority_Path2()
        {
            var target = new ObjectDescriptor("testScheme://testAuthority//testSegment");
            Assert.AreEqual("testScheme://testAuthority//testSegment", target.ToString());
            Assert.AreEqual("testscheme", target.Scheme);
            Assert.AreEqual("testauthority", target.MachineName);
            Assert.AreEqual("", target.ProgId);
            Assert.AreEqual("", target.ServerClass);
            Assert.AreEqual("", target.ObjectId);
            Assert.AreEqual("", target.ClsidString);
            Assert.AreEqual(Guid.Empty, target.Clsid);
        }

        [TestMethod]
        public void CtorTest_String_Scheme_Authority_Path2_Clsid()
        {
            var target = new ObjectDescriptor("testScheme://testAuthority//" + TestClsidServerClass);
            Assert.AreEqual("testScheme://testAuthority//" + TestClsidServerClass, target.ToString());
            Assert.AreEqual("testscheme", target.Scheme);
            Assert.AreEqual("testauthority", target.MachineName);
            Assert.AreEqual("", target.ProgId);
            Assert.AreEqual(TestClsidServerClass.ToLower(), target.ServerClass);
            Assert.AreEqual(TestClsidServerClass.ToLower(), target.ObjectId);
            Assert.AreEqual(TestClsidString.ToLower(), target.ClsidString);
            Assert.AreEqual(TestClsid, target.Clsid);
        }

        [TestMethod]
        public void CtorTest_String_Scheme_Authority_Path3()
        {
            var target = new ObjectDescriptor("testScheme://testAuthority/testSegment1/testSegment2");
            Assert.AreEqual("testScheme://testAuthority/testSegment1/testSegment2", target.ToString());
            Assert.AreEqual("testscheme", target.Scheme);
            Assert.AreEqual("testauthority", target.MachineName);
            Assert.AreEqual("testSegment1", target.ProgId);
            Assert.AreEqual("testSegment1", target.ServerClass);
            Assert.AreEqual("testSegment1", target.ObjectId);
            Assert.AreEqual("", target.ClsidString);
            Assert.AreEqual(Guid.Empty, target.Clsid);
        }

        [TestMethod]
        public void CtorTest_String_Scheme_Authority_Path3_Clsid()
        {
            var target = new ObjectDescriptor("testScheme://testAuthority/testSegment1/" + TestClsidServerClass);
            Assert.AreEqual("testScheme://testAuthority/testSegment1/" + TestClsidServerClass, target.ToString());
            Assert.AreEqual("testscheme", target.Scheme);
            Assert.AreEqual("testauthority", target.MachineName);
            Assert.AreEqual("testSegment1", target.ProgId);
            Assert.AreEqual("testSegment1", target.ServerClass);
            Assert.AreEqual("testSegment1/" + TestClsidServerClass.ToLower(), target.ObjectId);
            Assert.AreEqual(TestClsidString.ToLower(), target.ClsidString);
            Assert.AreEqual(TestClsid, target.Clsid);
        }

        [TestMethod]
        public void CtorTest_String_Scheme_Authority_Path4()
        {
            var target = new ObjectDescriptor("testScheme://testAuthority//testSegment1/testSegment2");
            Assert.AreEqual("testScheme://testAuthority//testSegment1/testSegment2", target.ToString());
            Assert.AreEqual("testscheme", target.Scheme);
            Assert.AreEqual("testauthority", target.MachineName);
            Assert.AreEqual("", target.ProgId);
            Assert.AreEqual("", target.ServerClass);
            Assert.AreEqual("", target.ObjectId);
            Assert.AreEqual("", target.ClsidString);
            Assert.AreEqual(Guid.Empty, target.Clsid);
        }

        [TestMethod]
        public void CtorTest_String_Scheme_Authority_Path4_Clsid()
        {
            var target = new ObjectDescriptor("testScheme://testAuthority//testSegment1/" + TestClsidServerClass);
            Assert.AreEqual("testScheme://testAuthority//testSegment1/" + TestClsidServerClass, target.ToString());
            Assert.AreEqual("testscheme", target.Scheme);
            Assert.AreEqual("testauthority", target.MachineName);
            Assert.AreEqual("", target.ProgId);
            Assert.AreEqual("", target.ServerClass);
            Assert.AreEqual("", target.ObjectId);
            Assert.AreEqual("", target.ClsidString);
            Assert.AreEqual(Guid.Empty, target.Clsid);
        }

        [TestMethod]
        public void CtorTest_String_Scheme_Authority_Path5()
        {
            var target = new ObjectDescriptor("testScheme://testAuthority/testSegment1/testSegment2/testSegment3");
            Assert.AreEqual("testScheme://testAuthority/testSegment1/testSegment2/testSegment3", target.ToString());
            Assert.AreEqual("testscheme", target.Scheme);
            Assert.AreEqual("testauthority", target.MachineName);
            Assert.AreEqual("testSegment1", target.ProgId);
            Assert.AreEqual("testSegment1", target.ServerClass);
            Assert.AreEqual("testSegment1", target.ObjectId);
            Assert.AreEqual("", target.ClsidString);
            Assert.AreEqual(Guid.Empty, target.Clsid);
        }

        [TestMethod]
        public void CtorTest_String_Scheme_Authority_Path5_Clsid()
        {
            var target = new ObjectDescriptor("testScheme://testAuthority/testSegment1/" + TestClsidServerClass + "/testSegment3");
            Assert.AreEqual("testScheme://testAuthority/testSegment1/" + TestClsidServerClass + "/testSegment3", target.ToString());
            Assert.AreEqual("testscheme", target.Scheme);
            Assert.AreEqual("testauthority", target.MachineName);
            Assert.AreEqual("testSegment1", target.ProgId);
            Assert.AreEqual("testSegment1", target.ServerClass);
            Assert.AreEqual("testSegment1/" + TestClsidServerClass.ToLower(), target.ObjectId);
            Assert.AreEqual(TestClsidString.ToLower(), target.ClsidString);
            Assert.AreEqual(TestClsid, target.Clsid);
        }

        [TestMethod]
        public void CtorTest_String_Scheme_Authority_Path6()
        {
            var target = new ObjectDescriptor("testScheme://testAuthority//testSegment1/testSegment2/testSegment3");
            Assert.AreEqual("testScheme://testAuthority//testSegment1/testSegment2/testSegment3", target.ToString());
            Assert.AreEqual("testscheme", target.Scheme);
            Assert.AreEqual("testauthority", target.MachineName);
            Assert.AreEqual("", target.ProgId);
            Assert.AreEqual("", target.ServerClass);
            Assert.AreEqual("", target.ObjectId);
            Assert.AreEqual("", target.ClsidString);
            Assert.AreEqual(Guid.Empty, target.Clsid);
        }

        [TestMethod]
        public void CtorTest_String_Scheme_Authority_Path6_Clsid()
        {
            var target = new ObjectDescriptor("testScheme://testAuthority//testSegment1/" + TestClsidServerClass + "/testSegment3");
            Assert.AreEqual("testScheme://testAuthority//testSegment1/" + TestClsidServerClass + "/testSegment3", target.ToString());
            Assert.AreEqual("testscheme", target.Scheme);
            Assert.AreEqual("testauthority", target.MachineName);
            Assert.AreEqual("", target.ProgId);
            Assert.AreEqual("", target.ServerClass);
            Assert.AreEqual("", target.ObjectId);
            Assert.AreEqual("", target.ClsidString);
            Assert.AreEqual(Guid.Empty, target.Clsid);
        }

        [TestMethod]
        public void CtorTest_String_String_Empty_Empty()
        {
            var target = new ObjectDescriptor("", "");
            Assert.AreEqual("dcom:", target.UrlString);
            Assert.AreEqual("dcom", target.Scheme);
            Assert.AreEqual("", target.MachineName);
            Assert.AreEqual("", target.ProgId);
            Assert.AreEqual("", target.ServerClass);
            Assert.AreEqual("", target.ObjectId);
            Assert.AreEqual("", target.ClsidString);
            Assert.AreEqual(Guid.Empty, target.Clsid);
        }

        [TestMethod]
        public void CtorTest_String_String_Empty_NotEmpty()
        {
            var target = new ObjectDescriptor("", "testServerClass");
            Assert.AreEqual("dcom:testServerClass", target.UrlString);
            Assert.AreEqual("dcom", target.Scheme);
            Assert.AreEqual("", target.MachineName);
            Assert.AreEqual("testServerClass", target.ProgId);
            Assert.AreEqual("testServerClass", target.ServerClass);
            Assert.AreEqual("testServerClass", target.ObjectId);
            Assert.AreEqual("", target.ClsidString);
            Assert.AreEqual(Guid.Empty, target.Clsid);
        }

        [TestMethod]
        public void CtorTest_String_String_Empty_NotEmpty_Clsid()
        {
            var target = new ObjectDescriptor("", TestClsidServerClass);
            Assert.AreEqual("dcom:" + Uri.EscapeDataString(TestClsidServerClass.ToLower()), target.UrlString);
            Assert.AreEqual("dcom", target.Scheme);
            Assert.AreEqual("", target.MachineName);
            Assert.AreEqual("", target.ProgId);
            Assert.AreEqual(TestClsidServerClass.ToLower(), target.ServerClass);
            Assert.AreEqual(TestClsidServerClass, target.ObjectId);
            Assert.AreEqual(TestClsidString.ToLower(), target.ClsidString);
            Assert.AreEqual(TestClsid, target.Clsid);
        }

        [TestMethod]
        public void CtorTest_String_String_NotEmpty_Empty()
        {
            var target = new ObjectDescriptor("machineName", "");
            Assert.AreEqual("dcom://machineName/", target.UrlString);
            Assert.AreEqual("dcom", target.Scheme);
            Assert.AreEqual("machineName", target.MachineName);
            Assert.AreEqual("", target.ProgId);
            Assert.AreEqual("", target.ServerClass);
            Assert.AreEqual("", target.ObjectId);
            Assert.AreEqual("", target.ClsidString);
            Assert.AreEqual(Guid.Empty, target.Clsid);
        }

        [TestMethod]
        public void CtorTest_String_String_NotEmpty_NotEmpty()
        {
            var target = new ObjectDescriptor("testMachineName", "testServerClass");
            Assert.AreEqual("dcom://testmachinename/testServerClass", target.UrlString);
            Assert.AreEqual("dcom", target.Scheme);
            Assert.AreEqual("testMachineName", target.MachineName);
            Assert.AreEqual("testServerClass", target.ProgId);
            Assert.AreEqual("testServerClass", target.ServerClass);
            Assert.AreEqual("testServerClass", target.ObjectId);
            Assert.AreEqual("", target.ClsidString);
            Assert.AreEqual(Guid.Empty, target.Clsid);
        }

        [TestMethod]
        public void CtorTest_String_String_NotEmpty_NotEmpty_Clsid()
        {
            var target = new ObjectDescriptor("testMachineName", TestClsidServerClass);
            Assert.AreEqual("dcom://testmachinename/" + Uri.EscapeDataString(TestClsidServerClass.ToLower()), target.UrlString);
            Assert.AreEqual("dcom", target.Scheme);
            Assert.AreEqual("testMachineName", target.MachineName);
            Assert.AreEqual("", target.ProgId);
            Assert.AreEqual(TestClsidServerClass.ToLower(), target.ServerClass);
            Assert.AreEqual(TestClsidServerClass, target.ObjectId);
            Assert.AreEqual(TestClsidString.ToLower(), target.ClsidString);
            Assert.AreEqual(TestClsid, target.Clsid);
        }

        [TestMethod]
        public void IsNullTest_Negative()
        {
            var target = new ObjectDescriptor("testMachineName");
            Assert.IsFalse(target.IsNull);
        }

        [TestMethod]
        public void IsNullTest_Positive()
        {
            var target = ObjectDescriptor.Null;
            Assert.IsTrue(target.IsNull);
        }

        [TestMethod]
        public void LocationTest_FromUri_1()
        {
            var target = new ObjectDescriptor { UrlString = "testScheme://testMachineName" };
            Assert.AreEqual("testmachinename", target.Location);
        }

        [TestMethod]
        public void LocationTest_FromUri_2()
        {
            var target = new ObjectDescriptor { UrlString = "testScheme://testMachineName/" };
            Assert.AreEqual("testmachinename", target.Location);
        }

        [TestMethod]
        public void LocationTest_FromUri_3()
        {
            var target = new ObjectDescriptor { UrlString = "testScheme://testMachineName/testPath" };
            Assert.AreEqual("testmachinename", target.Location);
        }

        [TestMethod]
        public void LocationTest_FromUri_4()
        {
            var target = new ObjectDescriptor
            {
                UrlString = "testScheme://testUser:testPassword@testMachineName/testPath"
            };
            Assert.AreEqual("testmachinename", target.Location);
        }

        [TestMethod]
        public void MachineNameTest_FromUri_1()
        {
            var target = new ObjectDescriptor { UrlString = "testScheme://testMachineName" };
            Assert.AreEqual("testmachinename", target.MachineName);
        }

        [TestMethod]
        public void MachineNameTest_FromUri_2()
        {
            var target = new ObjectDescriptor { UrlString = "testScheme://testMachineName/" };
            Assert.AreEqual("testmachinename", target.MachineName);
        }

        [TestMethod]
        public void MachineNameTest_FromUri_3()
        {
            var target = new ObjectDescriptor { UrlString = "testScheme://testMachineName/testPath" };
            Assert.AreEqual("testmachinename", target.MachineName);
        }

        [TestMethod]
        public void MachineNameTest_FromUri_4()
        {
            var target = new ObjectDescriptor
            {
                UrlString = "testScheme://testUser:testPassword@testMachineName/testPath"
            };
            Assert.AreEqual("testmachinename", target.MachineName);
        }

        [TestMethod]
        public void MachineNameTest_AssignAfterServerClass()
        {
            var target = new ObjectDescriptor { ServerClass = "testServerClass", MachineName = "testMachineName" };
            Assert.AreEqual("dcom://testMachineName/testServerClass", target.ToString());
            Assert.AreEqual("testMachineName", target.MachineName);
        }

        [TestMethod]
        public void MachineNameTest_AssignToNew()
        {
            var target = new ObjectDescriptor { MachineName = "testMachineName" };
            Assert.AreEqual("dcom://testMachineName/", target.ToString());
            Assert.AreEqual("testMachineName", target.MachineName);
        }

        [TestMethod]
        public void ServerClassTest_AssignAfterMachineName()
        {
            var target = new ObjectDescriptor { MachineName = "testMachineName", ServerClass = "testServerClass" };
            Assert.AreEqual("dcom://testmachinename/testServerClass", target.ToString());
            Assert.AreEqual("testServerClass", target.ServerClass);
        }

        [TestMethod]
        public void ServerClassTest_AssignAfterMachineName_Clsid()
        {
            var target = new ObjectDescriptor { MachineName = "testMachineName", ServerClass = TestClsidServerClass };
            Assert.AreEqual("dcom://testmachinename/" + TestClsidServerClass.ToLower(), target.ToString());
            Assert.AreEqual("dcom://testmachinename/" + Uri.EscapeDataString(TestClsidServerClass.ToLower()), target.UrlString);
            Assert.AreEqual(TestClsidServerClass, target.ServerClass);
        }

        [TestMethod]
        public void ServerClassTest_AssignToNew()
        {
            var target = new ObjectDescriptor { ServerClass = "testServerClass" };
            Assert.AreEqual("dcom:testServerClass", target.ToString());
            Assert.AreEqual("testServerClass", target.ServerClass);
        }

        [TestMethod]
        public void ServerClassTest_AssignToNew_Clsid()
        {
            var target = new ObjectDescriptor { ServerClass = TestClsidServerClass };
            Assert.AreEqual("dcom:" + TestClsidServerClass.ToLower(), target.ToString());
            Assert.AreEqual("dcom:" + Uri.EscapeDataString(TestClsidServerClass.ToLower()), target.UrlString);
            Assert.AreEqual(TestClsidServerClass, target.ServerClass);
        }

        [TestMethod]
        public void ServerClassTest_FromUri_1()
        {
            var target = new ObjectDescriptor { UrlString = "testScheme://" };
            Assert.AreEqual("", target.ServerClass);
        }

        [TestMethod]
        public void ServerClassTest_FromUri_2()
        {
            var target = new ObjectDescriptor { UrlString = "testScheme://testMachineName" };
            Assert.AreEqual("", target.ServerClass);
        }

        [TestMethod]
        public void ServerClassTest_FromUri_3()
        {
            var target = new ObjectDescriptor { UrlString = "testScheme://testMachineName/" };
            Assert.AreEqual("", target.ServerClass);
        }

        [TestMethod]
        public void ServerClassTest_FromUri_4()
        {
            var target = new ObjectDescriptor { UrlString = "testScheme:testServerClass" };
            Assert.AreEqual("testServerClass", target.ServerClass);
        }

        [TestMethod]
        public void ServerClassTest_FromUri_4_Clsid()
        {
            var target = new ObjectDescriptor { UrlString = "testScheme:" + TestClsidServerClass };
            Assert.AreEqual(TestClsidServerClass.ToLower(), target.ServerClass);
        }

        [TestMethod]
        // ReSharper disable once InconsistentNaming
        public void ServerClassTest_FromUri_4a()
        {
            var target = new ObjectDescriptor { UrlString = "testScheme:/testServerClass" };
            Assert.AreEqual("testServerClass", target.ServerClass);
        }

        [TestMethod]
        // ReSharper disable once InconsistentNaming
        public void ServerClassTest_FromUri_4a_Clsid()
        {
            var target = new ObjectDescriptor { UrlString = "testScheme:/" + TestClsidServerClass };
            Assert.AreEqual(TestClsidServerClass.ToLower(), target.ServerClass);
        }

        [TestMethod]
        public void ServerClassTest_FromUri_5()
        {
            var target = new ObjectDescriptor { UrlString = "testScheme:///testServerClass" };
            Assert.AreEqual("testServerClass", target.ServerClass);
        }

        [TestMethod]
        public void ServerClassTest_FromUri_5_Clsid()
        {
            var target = new ObjectDescriptor { UrlString = "testScheme:///" + TestClsidServerClass };
            Assert.AreEqual(TestClsidServerClass.ToLower(), target.ServerClass);
        }

        [TestMethod]
        // ReSharper disable once InconsistentNaming
        public void ServerClassTest_FromUri_5a()
        {
            var target = new ObjectDescriptor { UrlString = "testScheme:////testServerClass" };
            Assert.AreEqual("", target.ServerClass);
        }

        [TestMethod]
        // ReSharper disable once InconsistentNaming
        public void ServerClassTest_FromUri_5a_Clsid()
        {
            var target = new ObjectDescriptor { UrlString = "testScheme:////" + TestClsidServerClass };
            Assert.AreEqual(TestClsidServerClass.ToLower(), target.ServerClass);
        }

        [TestMethod]
        public void ServerClassTest_FromUri_6()
        {
            var target = new ObjectDescriptor { UrlString = "testScheme://testMachineName/testServerClass" };
            Assert.AreEqual("testServerClass", target.ServerClass);
        }

        [TestMethod]
        public void ServerClassTest_FromUri_6_Clsid()
        {
            var target = new ObjectDescriptor { UrlString = "testScheme://testMachineName/" + TestClsidServerClass };
            Assert.AreEqual(TestClsidServerClass.ToLower(), target.ServerClass);
        }

        [TestMethod]
        // ReSharper disable once InconsistentNaming
        public void ServerClassTest_FromUri_6a()
        {
            var target = new ObjectDescriptor { UrlString = "testScheme://testMachineName//testServerClass" };
            Assert.AreEqual("", target.ServerClass);
        }

        [TestMethod]
        // ReSharper disable once InconsistentNaming
        public void ServerClassTest_FromUri_6a_Clsid()
        {
            var target = new ObjectDescriptor { UrlString = "testScheme://testMachineName//" + TestClsidServerClass };
            Assert.AreEqual(TestClsidServerClass.ToLower(), target.ServerClass);
        }

        [TestMethod]
        public void ToStringTest_MachineNameAndServerClass()
        {
            var target = new ObjectDescriptor("testMachineName", "testServerClass");
            Assert.AreEqual("dcom://testmachinename/testServerClass", target.ToString());
        }

        [TestMethod]
        public void ToStringTest_MachineNameAndServerClass_Clsid()
        {
            var target = new ObjectDescriptor("testMachineName", TestClsidServerClass);
            Assert.AreEqual("dcom://testmachinename/" + TestClsidServerClass.ToLower(), target.ToString());
            Assert.AreEqual("dcom://testmachinename/" + Uri.EscapeDataString(TestClsidServerClass.ToLower()), target.UrlString);
        }

        [TestMethod]
        public void ToStringTest_MachineNameOnly()
        {
            var target = new ObjectDescriptor("testMachineName", "");
            Assert.AreEqual("dcom://testMachineName/", target.ToString());
        }

        [TestMethod]
        public void ToStringTest_Null()
        {
            var target = ObjectDescriptor.Null;
            Assert.AreEqual("", target.ToString());
        }

        [TestMethod]
        public void ToStringTest_ServerClassOnly()
        {
            var target = new ObjectDescriptor("testServerClass");
            Assert.AreEqual("dcom:testServerClass", target.ToString());
        }

        [TestMethod]
        public void ToStringTest_ServerClassOnly_Clsid()
        {
            var target = new ObjectDescriptor(TestClsidServerClass);
            Assert.AreEqual("dcom:" + TestClsidServerClass, target.ToString());
        }

        [TestMethod]
        public void EqualityTest_NewObjects()
        {
            var target1 = new ObjectDescriptor();
            var target2 = new ObjectDescriptor();
            Assert.AreEqual(target1.GetHashCode(), target2.GetHashCode());
            Assert.AreEqual(target1, target2);
        }

        [TestMethod]
        public void EqualityTest_Copy()
        {
            var target1 = new ObjectDescriptor("http://111.222.33.44/TestSegment1/TestSegment.2");
            AssertUtilities.WriteLine(target1);
            var target2 = new ObjectDescriptor(target1);
            AssertUtilities.WriteLine(target2);
            Assert.AreEqual(target1.GetHashCode(), target2.GetHashCode());
            Assert.AreEqual(target1, target2);
        }

        #endregion
    }
}
