﻿using System;
using System.Collections.Generic;
using System.Linq;
using OpcLabs.BaseLib.Forms.Testing;
using OpcLabs.BaseLib.Persistence;
using OpcLabs.EasyOpc.UA;
using OpcLabs.EasyOpc.UA.Engine;
using OpcLabs.EasyOpc.UA.Forms.Browsing;
using OpcLabs.EasyOpc.UA.InformationModel;
using OpcLabs.EasyOpc.UA.Reactive;

namespace OpcLabs.EasyOpcFormsTestApp.UA
{
    static class UACommands
    {
        static internal void BrowseControls()
        {
            var form = new UABrowseControlsForm();
            form.Show();
        }

        static internal void ChallengeTest()
        {
            var form = new ChallengeTestForm();
            form.Show();
        }

        static internal void ObjectTestbed()
        {
            var objectTypes = new List<Type>();
            objectTypes.AddRange(
                typeof(UAStatusCode).Assembly.GetTypes()
                .Concat(typeof(EasyUAClient).Assembly.GetTypes())
                .Concat(typeof(EasyUAClientConfiguration).Assembly.GetTypes())
                .Concat(typeof(UADataChangeNotificationObservable).Assembly.GetTypes())
                .Concat(typeof(UABrowseDialog).Assembly.GetTypes()));
            var favoriteObjectTypes = new List<Type>
                {
                    typeof (EasyUAClient),
                    typeof (UAModelClient)
                };
            Type primaryType = typeof(EasyUAClient);

            var form = new ObjectTestbedForm();
            form.SetOnlyUsableTypes(objectTypes);
            form.FavoriteTypes = favoriteObjectTypes;
            form.CurrentType = primaryType;

            var deferredInstanceInjector = new DeferredInstanceInjector(form.Instances);
            deferredInstanceInjector.AddDeferredInstance("ClientForOpcCompliance", typeof(EasyUAClient), () =>
                new EasyUAClient { Isolated = true, IsolatedParameters = EasyUAAdaptableParameters.OpcCompliance });
            deferredInstanceInjector.AddDeferredInstance("ClientForInteroperability", typeof(EasyUAClient), () =>
                new EasyUAClient { Isolated = true, IsolatedParameters = EasyUAAdaptableParameters.Interoperability });
            deferredInstanceInjector.AddDeferredInstance("IsolatedClient1", typeof(EasyUAClient), () =>
                new EasyUAClient { Isolated = true });
            deferredInstanceInjector.AddDeferredInstance("IsolatedClient2", typeof(EasyUAClient), () =>
                new EasyUAClient { Isolated = true });
            deferredInstanceInjector.AddDeferredInstance("ModelClientWithServerModel", typeof(UAModelClient),
                UAModelClient.CreateWithServerModel);
            form.Instances = deferredInstanceInjector;
            form.Instances.SaveInstance("EmptyArrayOfByte", new Byte[0]);
            form.Instances.SaveInstance("EmptyArrayOfInt16", new Int16[0]);
            form.Instances.SaveInstance("EmptyArrayOfInt32", new Int32[0]);
            form.Instances.SaveInstance("EmptyArrayOfInt64", new Int64[0]);
            form.Instances.SaveInstance("EmptyArrayOfDecimal", new Decimal[0]);
            form.Instances.SaveInstance("EmptyArrayOfSingle", new Single[0]);
            form.Instances.SaveInstance("EmptyArrayOfDouble", new Double[0]);
            form.Instances.SaveInstance("EmptyArrayOfBoolean", new Boolean[0]);
            form.Instances.SaveInstance("EmptyArrayOfString", new String[0]);

            form.InitialAction = ObjectTestbedFormAction.Instantiate;
            form.ImportedNamespaces = new[]
                {
                    "OpcLabs.BaseLib.Diagnostics",
                    "OpcLabs.EasyOpc.UA",
                    "OpcLabs.EasyOpc.UA.Modelling",
                    "System",
                    "System.Collections.Generic"
                };

            form.Show();
        }
    }
}
