﻿using System;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using OpcLabs.BaseLib.Runtime.InteropServices.Internal;
using OpcLabs.EasyOpc;
using OpcLabs.EasyOpc.DataAccess;
using OpcLabs.EasyOpc.DataAccess.Extensions;
using OpcLabs.EasyOpc.OperationModel;

namespace OpcLabs.EasyOpcLiveTest.DataAccess
{
    [TestClass]
    // ReSharper disable once InconsistentNaming
    public class EasyDAClientTest_Com_Native : EasyDAClientTest_Com
    {

        [TestCategory("Live")]
        [TestCategory("OpcClassic")]
        [TestCategory("OpcDataAccess")]
        [TestMethod]
        public void ReadItemTest_Com_Native_UnregisteredProgID()
        {
            IEasyDAClient target = CreateClient();
            try
            {
                target.ReadItem("", UnregisteredProgId, "Simulation.Constant_R4");
                Assert.Fail();
            }
            catch (OpcException opcException)
            {
                if (!target.SupportsTechnology(OpcTechnologies.Com))
                {
                    Assert.IsInstanceOfType(opcException.InnerException, typeof(NotSupportedException));
                    return;
                }

                Exception innerException = opcException.InnerException;
                Assert.IsNotNull(innerException);

                AssertBaseException(innerException,
                    unchecked((int)HRESULT.CO_E_CLASSSTRING),
                    "http://opcfoundation.org/DataAccess/",
                    "E_FAIL.0x800401F3");
            }
        }

#if (!NETFRAMEWORK)
        protected override bool ClientSupportsCom => false;
#endif

        protected override EasyDAClient CreateClient()
        {
            EasyDAClient client = base.CreateClient();
            client.InstanceParameters.EnableNativeClient = true;
            client.InstanceParameters.EnableNetApiClient = false;
            client.InstanceParameters.EnableServiceClient = false;
            return client;
        }
    }
}
