﻿
using System;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using OpcLabs.EasyOpc.UA;
using OpcLabs.EasyOpcLiveTest.UA.Environment;
using OpcLabs.UnitTestLib;

namespace OpcLabs.EasyOpcLiveTest.UA
{
    public partial class EasyUAClientTest
    {
        #region Test Methods

        [TestCategory("Live")]
        [TestCategory("OpcUA")]
        [TestMethod]
        public void CallMethodTest1()
        {
            TestSection.Arrange();

            UAEndpointDescriptor endpointDescriptor = UALiveTestEnvironment.OpcFoundation_UASampleServer.EndpointDescriptors.OpcTcp;
            TestEnvironment.PrecheckEndpoint(endpointDescriptor);

            var target = new EasyUAClient();
            var inputs = new object[]
                // ReSharper disable RedundantCast
                { false, (SByte)1, (Byte)2, (short)3, (ushort)4, (int)5, (uint)6, (long)7, (ulong)8, (float)9, (double)10 };
            // ReSharper restore RedundantCast
            object[] outputs = target.CallMethod(
                endpointDescriptor,
                OpcFoundation_UASampleServer.NodeIdExpandedTexts.Data_Static_MethodTest,
                OpcFoundation_UASampleServer.NodeIdExpandedTexts.Data_Static_MethodTest_ScalarMethod1,
                inputs);

            var expectedOutputs = new object[]
                // ReSharper disable RedundantCast
                { false, (Int16)1, (Byte)2, (short)3, (Int32)4, (int)5, (Int64)6, (long)7, (Decimal)8, (float)9, (double)10 };
            // ReSharper restore RedundantCast

            Assert.IsNotNull(outputs);
            CollectionAssert.AreEqual(outputs, expectedOutputs);
        }

        [TestCategory("Live")]
        [TestCategory("OpcUA")]
        [TestMethod]
        public void CallMethodTest2()
        {
            TestSection.Arrange();

            UAEndpointDescriptor endpointDescriptor = UALiveTestEnvironment.OpcFoundation_UASampleServer.EndpointDescriptors.OpcTcp;
            TestEnvironment.PrecheckEndpoint(endpointDescriptor);

            var target = new EasyUAClient();
            var inputs = new object[] { false, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10 };
            var types = new[] { typeof(bool), typeof(SByte), typeof(Byte), typeof(short), typeof(ushort), typeof(int), typeof(uint), typeof(long), typeof(ulong), typeof(float), typeof(double) };
            object[] outputs = target.CallMethod(
                endpointDescriptor,
                OpcFoundation_UASampleServer.NodeIdExpandedTexts.Data_Static_MethodTest,
                OpcFoundation_UASampleServer.NodeIdExpandedTexts.Data_Static_MethodTest_ScalarMethod1,
                inputs,
                types);

            var expectedOutputs = new object[]
                // ReSharper disable RedundantCast
                { false, (Int16)1, (Byte)2, (short)3, (Int32)4, (int)5, (Int64)6, (long)7, (Decimal)8, (float)9, (double)10 };
            // ReSharper restore RedundantCast

            Assert.IsNotNull(outputs);
            CollectionAssert.AreEqual(outputs, expectedOutputs);
        }

        #endregion
    }
}
