﻿
using JetBrains.Annotations;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using OpcLabs.BaseLib.DataTypeModel;
using OpcLabs.EasyOpc.UA.DataTypeModel;

namespace OpcLabs.EasyOpcTest.UA.DataTypeModel.Internal
{
    partial class TypeDescriptionExamplesTest
    {
        #region Static Private Properties

        [NotNull]
        static private StructuredDataType IntegerArrayDataType
        {
            get
            {
                var integerArrayDataType = new StructuredDataType("IntegerArray", "TypeDescriptionExamples:IntegerArray")
                {
                    new DataField("Size", UAOpcBinaryStandardDataTypes.Int32) {IsLength = true},
                    new DataField("Array", new SequenceDataType(UAOpcBinaryStandardDataTypes.Int32)) {IsOptional=true}
                };
                integerArrayDataType.Description = "An array of integers prefixed by its length.";
                return integerArrayDataType;
            }
        }

        [NotNull]
        static private StructuredData IntegerArray_0Data => new StructuredData(IntegerArrayDataType)
        {
            {"Size", new PrimitiveData(UAOpcBinaryStandardDataTypes.Int32, 0)},
            {"Array", new SequenceData(new SequenceDataType(UAOpcBinaryStandardDataTypes.Int32)) }
        };

        [NotNull]
        static private byte[] IntegerArray_0ExtensionObjectBody => new byte[4];

        [NotNull]
        static private StructuredData IntegerArray_1Data => new StructuredData(IntegerArrayDataType)
        {
            {"Size", new PrimitiveData(UAOpcBinaryStandardDataTypes.Int32, 1)},
            {
                "Array", new SequenceData(new SequenceDataType(UAOpcBinaryStandardDataTypes.Int32))
                {
                    new PrimitiveData(UAOpcBinaryStandardDataTypes.Int32, 11)
                }
            }
        };

        [NotNull]
        static private byte[] IntegerArray_1ExtensionObjectBody => new byte[] { 1, 0, 0, 0, 11, 0, 0, 0 };

        [NotNull]
        static private StructuredData IntegerArray_3Data => new StructuredData(IntegerArrayDataType)
        {
            {"Size", new PrimitiveData(UAOpcBinaryStandardDataTypes.Int32, 3)},
            {
                "Array", new SequenceData(new SequenceDataType(UAOpcBinaryStandardDataTypes.Int32))
                {
                    new PrimitiveData(UAOpcBinaryStandardDataTypes.Int32, 11),
                    new PrimitiveData(UAOpcBinaryStandardDataTypes.Int32, 22),
                    new PrimitiveData(UAOpcBinaryStandardDataTypes.Int32, 33)
                }
            }
        };

        [NotNull]
        static private byte[] IntegerArray_3ExtensionObjectBody => new byte[] { 3, 0, 0, 0, 11, 0, 0, 0, 22, 0, 0, 0, 33, 0, 0, 0 };

        [NotNull]
        static private StructuredData IntegerArray_Minus1Data => new StructuredData(IntegerArrayDataType)
        {
            {"Size", new PrimitiveData(UAOpcBinaryStandardDataTypes.Int32, -1)}
        };

        [NotNull]
        static private byte[] IntegerArray_Minus1ExtensionObjectBody => new byte[] { 0xFF, 0xFF, 0xFF, 0xFF };

        [NotNull]
        static private StructuredData IntegerArray_Minus2Data => new StructuredData(IntegerArrayDataType)
        {
            {"Size", new PrimitiveData(UAOpcBinaryStandardDataTypes.Int32, -2)}
        };

        [NotNull]
        static private byte[] IntegerArray_Minus2ExtensionObjectBody => new byte[] { 0xFE, 0xFF, 0xFF, 0xFF };

        #endregion



        #region Test Methods

        [TestMethod]
        public void DecodeTest_IntegerArray_0() =>
            DecodeTest(null, TypeDictionaryByteArray, "IntegerArray", IntegerArray_0ExtensionObjectBody, IntegerArray_0Data);

        [TestMethod]
        public void DecodeTest_IntegerArray_1() =>
            DecodeTest(null, TypeDictionaryByteArray, "IntegerArray", IntegerArray_1ExtensionObjectBody, IntegerArray_1Data);

        [TestMethod]
        public void DecodeTest_IntegerArray_3() =>
            DecodeTest(null, TypeDictionaryByteArray, "IntegerArray", IntegerArray_3ExtensionObjectBody, IntegerArray_3Data);

        [TestMethod]
        public void DecodeTest_IntegerArray_Minus1() =>
            DecodeTest(null, TypeDictionaryByteArray, "IntegerArray", IntegerArray_Minus1ExtensionObjectBody, IntegerArray_Minus1Data);

        [TestMethod]
        public void DecodeTest_IntegerArray_Minus2() =>
            DecodeTest(null, TypeDictionaryByteArray, "IntegerArray", IntegerArray_Minus2ExtensionObjectBody, IntegerArray_Minus2Data);

        [TestMethod]
        public void EncodeTest_IntegerArray_0() =>
            EncodeTest(null, TypeDictionaryByteArray, "IntegerArray", IntegerArray_0Data, IntegerArray_0ExtensionObjectBody);

        [TestMethod]
        public void EncodeTest_IntegerArray_1() =>
            EncodeTest(null, TypeDictionaryByteArray, "IntegerArray", IntegerArray_1Data, IntegerArray_1ExtensionObjectBody);

        [TestMethod]
        public void EncodeTest_IntegerArray_3() =>
            EncodeTest(null, TypeDictionaryByteArray, "IntegerArray", IntegerArray_3Data, IntegerArray_3ExtensionObjectBody);

        [TestMethod]
        public void EncodeTest_IntegerArray_Minus1() =>
            EncodeTest(null, TypeDictionaryByteArray, "IntegerArray", IntegerArray_Minus1Data, IntegerArray_Minus1ExtensionObjectBody);

        [TestMethod]
        public void EncodeTest_IntegerArray_Minus2() =>
            EncodeTest(null, TypeDictionaryByteArray, "IntegerArray", IntegerArray_Minus2Data, IntegerArray_Minus2ExtensionObjectBody);

        #endregion
    }
}
